@@ -191,7 +191,6 @@ goc_polyline_distance (GocItem *item, double x, double y, GocItem **near_item)
 {
 	GocPolyline *polyline = GOC_POLYLINE (item);
 	GOStyle *style = go_style_dup (go_styled_object_get_style (GO_STYLED_OBJECT (item)));
-	double tmp_width = 0;
 	double res = 20;
 	double ppu = goc_canvas_get_pixels_per_unit (item->canvas);
 	cairo_surface_t *surface;
@@ -201,7 +200,6 @@ goc_polyline_distance (GocItem *item, double x, double y, GocItem **near_item)
 		return res;
 
 	*near_item = item;
-	tmp_width = style->line.width;
 	if (style->line.width * ppu < 5)
 		style->line.width = 5. / (ppu * ppu);
 	else
@@ -217,7 +215,6 @@ goc_polyline_distance (GocItem *item, double x, double y, GocItem **near_item)
 	g_object_unref (style);
 	cairo_destroy (cr);
 	cairo_surface_destroy (surface);
-	style->line.width = tmp_width;
 	return res;
 }
 
