@@ -48,7 +48,7 @@ struct _GOColorPalette {
 	gboolean      allow_alpha;
 
 	/* only for custom colours */
-	GtkWidget   *swatches [GO_COLOR_GROUP_HISTORY_SIZE];
+	GtkWidget   *swatches[GO_COLOR_GROUP_HISTORY_SIZE];
 
 	/* The table with our default color names */
 	GONamedColor const *default_set;
@@ -66,6 +66,8 @@ typedef struct {
 #define COLOR_PREVIEW_WIDTH 12
 #define COLOR_PREVIEW_HEIGHT 12
 
+#define CCW_KEY "GOColorPalette::ccw"
+
 enum {
 	COLOR_CHANGED,
 	DISPLAY_CUSTOM_DIALOG,
@@ -153,36 +155,52 @@ static guint go_color_palette_signals [LAST_SIGNAL] = { 0, };
 static GObjectClass *go_color_palette_parent_class;
 
 static GtkWidget *
-create_color_sel (GObject *action_proxy, GOColor c, GCallback handler, gboolean allow_alpha)
+create_color_sel (GtkWidget *parent_widget, GObject *action_proxy,
+		  GOColor c, GCallback handler, gboolean allow_alpha)
 {
-	char *title = g_object_get_data (G_OBJECT (action_proxy), "title");
-	GtkWidget *w = gtk_color_selection_dialog_new (title), *hb;
-	GtkColorSelectionDialog *dialog = GTK_COLOR_SELECTION_DIALOG (w);
-	GtkColorSelection *colorsel = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (dialog));
+	const char *title =
+		g_object_get_data (G_OBJECT (action_proxy), "title");
+	GtkWidget *toplevel = gtk_widget_get_toplevel (parent_widget);
+	GtkWidget *dialog = gtk_dialog_new_with_buttons
+		(title,
+		 (gtk_widget_is_toplevel (toplevel)
+		  ? GTK_WINDOW (toplevel)
+		  : NULL),
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_STOCK_OK, GTK_RESPONSE_OK,
+		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		 NULL);
+	GtkWidget *ccw = gtk_color_chooser_widget_new ();
+	GtkWidget *dca = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 	GdkRGBA gdk;
 
-	g_object_get (G_OBJECT (w), "help-button", &hb, NULL);
-	gtk_widget_hide (hb);
-	gtk_color_selection_set_has_opacity_control (colorsel, allow_alpha);
-	gtk_color_selection_set_current_rgba (colorsel,
-		go_color_to_gdk_rgba (c, &gdk));
+	g_object_set_data (G_OBJECT (dialog), CCW_KEY, ccw);
+
+	gtk_container_add (GTK_CONTAINER (dca), ccw);
+
+	go_color_to_gdk_rgba (c, &gdk);
+
+	g_object_set (G_OBJECT (ccw),
+		      "use-alpha", allow_alpha,
+		      "rgba", &gdk,
+		      "show-editor", TRUE,
+		      NULL);
 
 	g_signal_connect_object (dialog,
 		"response", handler, action_proxy, 0);
 
 	/* require an explicit show _after_ the custom-dialog signal fires */
-	return w;
+	return dialog;
 }
 
 static gboolean
-handle_color_sel (GtkColorSelectionDialog *dialog,
+handle_color_sel (GtkDialog *dialog,
 		  gint response_id, GOColor *res)
 {
 	if (response_id == GTK_RESPONSE_OK) {
+		GtkColorChooser *chooser = g_object_get_data (G_OBJECT (dialog), CCW_KEY);
 		GdkRGBA gdk;
-		GtkColorSelection *colorsel = GTK_COLOR_SELECTION (gtk_color_selection_dialog_get_color_selection (dialog));
-
-		gtk_color_selection_get_current_rgba (colorsel, &gdk);
+		gtk_color_chooser_get_rgba (chooser, &gdk);
 		*res = GO_COLOR_FROM_GDK_RGBA (gdk);
 	}
 	/* destroy _before_ we emit */
@@ -224,12 +242,17 @@ go_color_palette_class_init (GObjectClass *gobject_class)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__OBJECT,
 			      G_TYPE_NONE, 1, G_TYPE_OBJECT);
+
+#ifdef HAVE_GTK_WIDGET_CLASS_SET_CSS_NAME
+	gtk_widget_class_set_css_name ((GtkWidgetClass *)gobject_class, "colorpalette");
+#endif
 }
 
 static void
 go_color_palette_init (GObject *obj)
 {
 	g_object_set (obj, "orientation", GTK_ORIENTATION_VERTICAL, NULL);
+	_go_gtk_widget_add_css_provider (GTK_WIDGET (obj));
 }
 
 GSF_CLASS (GOColorPalette, go_color_palette,
@@ -271,19 +294,12 @@ static void
 cb_history_changed (GOColorPalette *pal)
 {
 	int i;
-	GdkRGBA gdk;
 	GOColorGroup *group = pal->group;
 
 	for (i = 0 ; i < GO_COLOR_GROUP_HISTORY_SIZE ; i++)
-		gtk_widget_override_background_color (pal->swatches [i], GTK_STATE_FLAG_NORMAL,
-			go_color_to_gdk_rgba (group->history[i], &gdk));
-#if 0
-	if (next_swatch != NULL) {
-		next_swatch->style->bg[GTK_STATE_NORMAL] = *new_color;
-		gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (pal->picker),
-			new_color->red, new_color->green, new_color->blue, 0);
-	}
-#endif
+		g_object_set_data (G_OBJECT (pal->swatches[i]),
+				   "color",
+				   GUINT_TO_POINTER (group->history[i]));
 }
 
 static gboolean
@@ -298,17 +314,15 @@ swatch_activated (GOColorPalette *pal, GtkBin *button)
 {
 	GList *tmp = gtk_container_get_children (GTK_CONTAINER (gtk_bin_get_child (button)));
 	GtkWidget *swatch = (tmp != NULL) ? tmp->data : NULL;
-	GtkStyleContext *style_ctx;
-	GdkRGBA rgba;
+	GOColor color;
 
 	g_list_free (tmp);
 
 	g_return_if_fail (swatch != NULL);
 
-	style_ctx = gtk_widget_get_style_context (swatch);
-	gtk_style_context_get_background_color (style_ctx, GTK_STATE_FLAG_NORMAL, &rgba);
-	set_color (pal, GO_COLOR_FROM_GDK_RGBA (rgba),
-		   FALSE, TRUE, FALSE);
+	color = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (swatch),
+						     "color"));
+	set_color (pal, color, FALSE, TRUE, FALSE);
 }
 
 static gboolean
@@ -335,7 +349,8 @@ static gboolean
 draw_color_cb (GtkWidget *widget, cairo_t *cr, gpointer data)
 {
 	GtkAllocation allocation;
-	GOColor color = GPOINTER_TO_UINT (data);
+	GOColor color = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (widget),
+							     "color"));
 	gtk_widget_get_allocation (widget, &allocation);
 	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (color));
 	cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
@@ -354,18 +369,15 @@ go_color_palette_button_new (GOColorPalette *pal, GtkGrid *grid,
 			     gint col, gint row)
 {
 	GtkWidget *button, *swatch, *box;
-	GdkRGBA   gdk;
 
 	swatch = gtk_drawing_area_new ();
 	g_signal_connect (G_OBJECT (swatch), "draw", G_CALLBACK (draw_color_cb),
-	                  GUINT_TO_POINTER (color_name->color));
-	gtk_widget_override_background_color (swatch, GTK_STATE_FLAG_NORMAL,
-	        go_color_to_gdk_rgba (color_name->color, &gdk));
-	gtk_widget_set_size_request (swatch, COLOR_PREVIEW_WIDTH, COLOR_PREVIEW_HEIGHT);
+	                  NULL);
+	g_object_set_data (G_OBJECT (swatch), "color",
+			   GUINT_TO_POINTER (color_name->color));
 
 	/* Wrap inside a vbox with a border so that we can see the focus indicator */
 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (box), 2);
 	gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (swatch), TRUE, TRUE, 0);
 
 	button = gtk_button_new ();
@@ -379,11 +391,19 @@ go_color_palette_button_new (GOColorPalette *pal, GtkGrid *grid,
 		"signal::button_release_event", G_CALLBACK (cb_swatch_release_event), pal,
 		"signal::key_press_event", G_CALLBACK (cb_swatch_key_press), pal,
 		NULL);
+
+	if (!_go_gtk_new_theming ()) {
+		gtk_widget_set_size_request (swatch, COLOR_PREVIEW_WIDTH, COLOR_PREVIEW_HEIGHT);
+		gtk_widget_set_halign (swatch, GTK_ALIGN_FILL);
+		gtk_widget_set_valign (swatch, GTK_ALIGN_FILL);
+		gtk_container_set_border_width (GTK_CONTAINER (box), 2);
+	}
+
 	return swatch;
 }
 
 static void
-cb_combo_custom_response (GtkColorSelectionDialog *dialog,
+cb_combo_custom_response (GtkDialog *dialog,
 			  gint response_id, GOColorPalette *pal)
 {
 	GOColor c;
@@ -394,11 +414,14 @@ cb_combo_custom_response (GtkColorSelectionDialog *dialog,
 static void
 cb_combo_custom_clicked (GtkWidget *button, GOColorPalette *pal)
 {
-	GtkWidget *dialog = create_color_sel (G_OBJECT (pal), pal->selection,
-		G_CALLBACK (cb_combo_custom_response), pal->allow_alpha);
+	GtkWidget *dialog = create_color_sel
+		(button,
+		 G_OBJECT (pal), pal->selection,
+		 G_CALLBACK (cb_combo_custom_response),
+		 pal->allow_alpha);
 	g_signal_emit (pal, go_color_palette_signals [DISPLAY_CUSTOM_DIALOG], 0,
 		dialog);
-	gtk_widget_show (dialog);
+	gtk_widget_show_all (dialog);
 }
 
 void
@@ -472,7 +495,7 @@ custom_colors :
 	for (col = 0; col < cols && col < GO_COLOR_GROUP_HISTORY_SIZE; col++) {
 		GONamedColor color_name = { 0, N_("custom") };
 		color_name.color = pal->group->history [col];
-		pal->swatches [col] = go_color_palette_button_new (pal,
+		pal->swatches[col] = go_color_palette_button_new (pal,
 			GTK_GRID (grid),
 			&color_name, col, row + 1);
 	}
@@ -650,7 +673,7 @@ cb_menu_color_activate (GtkWidget *button, GOMenuColor *menu)
 		       color, FALSE, TRUE, FALSE);
 }
 static void
-cb_menu_custom_response (GtkColorSelectionDialog *dialog,
+cb_menu_custom_response (GtkDialog *dialog,
 			 gint response_id, GOMenuColor *menu)
 {
 	GOColor c;
@@ -664,11 +687,14 @@ cb_menu_custom_response (GtkColorSelectionDialog *dialog,
 static void
 cb_menu_custom_activate (GtkWidget *button, GOMenuColor *menu)
 {
-	GtkWidget *dialog = create_color_sel (G_OBJECT (menu), menu->selection,
-		G_CALLBACK (cb_menu_custom_response), menu->allow_alpha);
-	g_signal_emit (menu, go_menu_color_signals [DISPLAY_CUSTOM_DIALOG], 0,
+	GtkWidget *dialog = create_color_sel
+		(button,
+		 G_OBJECT (menu), menu->selection,
+		 G_CALLBACK (cb_menu_custom_response),
+		 menu->allow_alpha);
+	g_signal_emit (menu, go_menu_color_signals[DISPLAY_CUSTOM_DIALOG], 0,
 		dialog);
-	gtk_widget_show (dialog);
+	gtk_widget_show_all (dialog);
 }
 
 /**
