@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <goffice/goffice-config.h>
@@ -1767,6 +1766,10 @@ nfs_class_init (GObjectClass *klass)
 			      accumulate_first_string, NULL,
 			      go__STRING__POINTER,
 			      G_TYPE_STRING, 1, G_TYPE_POINTER);
+
+#ifdef HAVE_GTK_WIDGET_CLASS_SET_CSS_NAME
+	gtk_widget_class_set_css_name ((GtkWidgetClass *)klass, "formatselector");
+#endif
 }
 
 GSF_CLASS (GOFormatSel, go_format_sel,
