@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <goffice/goffice-config.h>
@@ -34,9 +33,17 @@ struct _GORotationSel {
 	GocItem		*line;
 	GtkWidget       *text_widget;
 	GocItem		*text;
-	int		 rot_width, rot_height;
+	int		 rot_width0, rot_width1;
 	gulong		 motion_handle;
 	gboolean full;
+
+	// Styling
+	GOColor          tick_color[2][2];         // [selected][major]
+	GOColor          tick_outline_color[2][2]; // [selected][major]
+	double           tick_outline_width[2][2]; // [selected][major]
+	double           tick_size[2][2];          // [selected][major]
+	GOColor          line_color;
+	double           line_width;
 };
 
 typedef struct {
@@ -52,10 +59,75 @@ enum {
 static guint grs_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *grs_parent_class;
 
+static void
+grs_reload_style (GORotationSel *grs)
+{
+	int i;
+#if GTK_CHECK_VERSION(3,20,0)
+	GtkStyleContext *ctxt_canvas =
+		gtk_widget_get_style_context (GTK_WIDGET (grs->rotate_canvas));
+#endif
+
+	for (i = 0; i < 4; i++) {
+		gboolean qSelected = (i & 1);
+		gboolean qMajor = ((i >> 1) & 1);
+#if GTK_CHECK_VERSION(3,20,0)
+		GtkStyleContext *ctxt = go_style_context_from_selector
+			(ctxt_canvas, qSelected ? "ticks:selected" : "ticks");
+		GdkRGBA *c, *oc;
+		int ow, mw;
+
+		gtk_style_context_add_class (ctxt,
+					     qMajor ? "major" : "minor");
+		gtk_style_context_get (ctxt,
+				       gtk_style_context_get_state (ctxt),
+				       "color", &c,
+				       "outline-color", &oc,
+				       "outline-width", &ow,
+				       "min-width", &mw,
+				       NULL);
+		grs->tick_color[qSelected][qMajor] = GO_COLOR_FROM_GDK_RGBA (*c);
+		grs->tick_outline_color[qSelected][qMajor] = GO_COLOR_FROM_GDK_RGBA (*oc);
+		grs->tick_outline_width[qSelected][qMajor] = ow;
+		// We highjack min-width to mean 2*radius
+		grs->tick_size[qSelected][qMajor] = mw / 2.0;
+		gdk_rgba_free (c);
+		gdk_rgba_free (oc);
+		g_object_unref (ctxt);
+#else
+		grs->tick_color[qSelected][qMajor] = GO_COLOR_BLACK;
+		grs->tick_outline_color[qSelected][qMajor] = GO_COLOR_BLACK;
+		grs->tick_outline_width[qSelected][qMajor] = 1;
+		grs->tick_size[qSelected][qMajor] = (qMajor ? 4 : 3);
+#endif
+	}
+
+#if GTK_CHECK_VERSION(3,20,0)
+	{
+		GtkStyleContext *ctxt = go_style_context_from_selector
+			(ctxt_canvas, "line");
+		GdkRGBA *c;
+		int ow;
+		gtk_style_context_get (ctxt,
+				       gtk_style_context_get_state (ctxt),
+				       "color", &c,
+				       "outline-width", &ow,
+				       NULL);
+		grs->line_color = GO_COLOR_FROM_GDK_RGBA (*c);
+		grs->line_width = ow;
+		gdk_rgba_free (c);
+		g_object_unref (ctxt);
+	}
+#else
+	grs->line_color = GO_COLOR_BLACK;
+	grs->line_width = 2;
+#endif
+}
+
+
 static void
 cb_rotate_changed (GORotationSel *grs)
 {
-	GOColor colour;
 	int i, maxi = grs->full? 23: 12;
 	double x0 = grs->full? 100.: 15.;
 	int angle;
@@ -64,23 +136,42 @@ cb_rotate_changed (GORotationSel *grs)
 		gtk_spin_button_get_value_as_int (grs->rotate_spinner) % 360);
 	angle = (grs->angle < -90)? grs->angle + 360: grs->angle;
 
-	for (i = 0 ; i <= maxi ; i++)
-		if (grs->rotate_marks[i] != NULL) {
-			GOStyle *style = go_styled_object_get_style (GO_STYLED_OBJECT (grs->rotate_marks[i]));
-			colour = (angle == (i-6)*15) ? GO_COLOR_GREEN : GO_COLOR_BLACK;
-			if (style->fill.pattern.back != colour){
-				style->fill.pattern.back = colour;
-				goc_item_invalidate (grs->rotate_marks[i]);
+	for (i = 0 ; i <= maxi ; i++) {
+		int mark_angle = (i - 6) * 15;
+		gboolean qSelected = (angle == mark_angle);
+		gboolean qMajor = (i % 3 == 0);
+		GocItem *item = grs->rotate_marks[i];
+
+		if (item != NULL) {
+			GOStyledObject *so = GO_STYLED_OBJECT (item);
+			GOStyle *style = go_styled_object_get_style (so);
+			GOColor color = grs->tick_color[qSelected][qMajor];
+			GOColor outline_color = grs->tick_outline_color[qSelected][qMajor];
+			int outline_width = grs->tick_outline_width[qSelected][qMajor];
+			double radius = grs->tick_size[qSelected][qMajor];
+			if (style->fill.pattern.back != color ||
+			    style->line.color != outline_color ||
+			    style->line.width != outline_width) {
+				style->line.color = outline_color;
+				style->line.width = outline_width;
+				style->fill.pattern.back = color;
+				go_styled_object_style_changed (so);
+			}
+			if (GOC_CIRCLE (item)->radius != radius) {
+				g_object_set (item, "radius", radius, NULL);
+				goc_item_invalidate (item);
 			}
 		}
+	}
 
 	if (grs->line != NULL) {
-		double rad = grs->angle * M_PI / 180.;
+		double c = go_cospi (grs->angle / 180.);
+		double s = go_sinpi (grs->angle / 180.);
 		goc_item_set (grs->line,
-		              "x0", x0 + cos (rad) * grs->rot_width,
-		              "y0", 100 - sin (rad) * grs->rot_width,
-		              "x1", x0 + cos (rad) * 72.,
-		              "y1", 100 - sin (rad) * 72.,
+		              "x0",  x0 + c * grs->rot_width0,
+		              "y0", 100 - s * grs->rot_width0,
+		              "x1",  x0 + c * grs->rot_width1,
+		              "y1", 100 - s * grs->rot_width1,
 		              NULL);
 	}
 
@@ -99,32 +190,26 @@ cb_rotate_canvas_realize (GocCanvas *canvas, GORotationSel *grs)
 	GocGroup  *group = goc_canvas_get_root (canvas);
 	int i, maxint = grs->full? 23: 12;
 	GOStyle *go_style;
-	GdkRGBA color = {1., 1., 1., 1.};
 	double x0 = grs->full? 100.: 15.;
+	double mov = 0;
 
-	gtk_widget_override_background_color (GTK_WIDGET (canvas),
-	                                      GTK_STATE_FLAG_NORMAL, &color);
-
-	for (i = 0 ; i <= maxint ; i++) {
-		double rad = (i-6) * M_PI / 12.;
-		double x = x0 + cos (rad) * 80.;
-		double y = 100 - sin (rad) * 80.;
-		double size = (i % 3) ? 3.0 : 4.0;
+	for (i = 0 ; i <= maxint; i++) {
+		int mark_angle = (i - 6) * 15;
+		double x = x0 + go_cospi (mark_angle / 180.0) * 80.;
+		double y = 100 - go_sinpi (mark_angle / 180.0) * 80.;
+		gboolean qMajor = (i % 3 == 0);
+		double size = grs->tick_size[FALSE][qMajor];
 		GocItem *item = goc_item_new (group,
 			GOC_TYPE_CIRCLE,
 			"x", x,	"y", y,
 			"radius", size,
 			NULL);
-		go_style = go_styled_object_get_style (GO_STYLED_OBJECT (item));
-		go_style->line.width = 1.;
-		go_style->line.color = GO_COLOR_BLACK;
-		go_style->fill.pattern.back = GO_COLOR_BLACK;
 		grs->rotate_marks[i] = item;
 	}
 	grs->line = goc_item_new (group, GOC_TYPE_LINE, NULL);
 	go_style = go_styled_object_get_style (GO_STYLED_OBJECT (grs->line));
-	go_style->line.width = 2.;
-	go_style->line.color = GO_COLOR_BLACK;
+	go_style->line.width = grs->line_width;
+	go_style->line.color = grs->line_color;
 
 	{
 		double x, y, w, h;
@@ -147,10 +232,18 @@ cb_rotate_canvas_realize (GocCanvas *canvas, GORotationSel *grs)
 		goc_item_get_bounds (grs->text, &x, &y, &w, &h);
 		pango_attr_list_unref (attrs);
 
-		grs->rot_width  = w - x;
-		grs->rot_height = h - y;
+		grs->rot_width0  = w - x;
 	}
 
+	for (i = 0; i < 4; i++) {
+		gboolean qSelected = (i & 1);
+		gboolean qMajor = ((i >> 1) & 1);
+		double ov = grs->tick_outline_width[qSelected][qMajor];
+		if (ov > mov)
+			mov = ov;
+	}
+	grs->rot_width1 = 73 - mov;
+
 	cb_rotate_changed (grs);
 }
 
@@ -161,7 +254,7 @@ set_rot_from_point (GORotationSel *grs, double x, double y)
 	x -= grs->full? 100.: 15.;	if (!grs->full && x < 0.) x = 0.;
 	y -= 100.;
 
-	degrees = atan2 (-y, x) * 180 / M_PI;
+	degrees = go_atan2pi (-y, x) * 180;
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (grs->rotate_spinner),
 		go_fake_round (degrees));
@@ -237,6 +330,11 @@ grs_init (GORotationSel *grs)
 
 	w = go_gtk_builder_get_widget (grs->gui, "toplevel");
 	gtk_grid_attach (GTK_GRID (grs), w, 0, 0, 1, 1);
+
+	_go_gtk_widget_add_css_provider (GTK_WIDGET (grs->rotate_canvas));
+
+	grs_reload_style (grs);
+
 	gtk_widget_show_all (GTK_WIDGET (grs));
 }
 
@@ -256,8 +354,7 @@ grs_finalize (GObject *obj)
 static void
 grs_class_init (GObjectClass *klass)
 {
-	GObjectClass *gobj_class = (GObjectClass *) klass;
-	gobj_class->finalize = grs_finalize;
+	klass->finalize = grs_finalize;
 
 	grs_parent_class = g_type_class_peek (gtk_box_get_type ());
 	grs_signals [ROTATION_CHANGED] = g_signal_new ("rotation-changed",
@@ -266,6 +363,10 @@ grs_class_init (GObjectClass *klass)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__INT,
 		G_TYPE_NONE, 1, G_TYPE_INT);
+
+#ifdef HAVE_GTK_WIDGET_CLASS_SET_CSS_NAME
+	gtk_widget_class_set_css_name ((GtkWidgetClass *)klass, "rotationselector");
+#endif
 }
 
 GSF_CLASS (GORotationSel, go_rotation_sel,
