@@ -1026,8 +1026,9 @@ go_object_properties_apply (GObject *obj, GSList *props, gboolean changed_only)
 			g_object_get_property (obj, pspec->name, &current);
 			doit = g_param_values_cmp (pspec, &current, value);
 #if 0
-			g_print ("%2d:  old: [%s]   new: [%s]\n",
+			g_print ("%2d:  %-20s  old: [%s]   new: [%s]\n",
 				 g_param_values_cmp (pspec, &current, value),
+				 g_param_spec_get_name (pspec),
 				 g_strdup_value_contents (value),
 				 g_strdup_value_contents (&current));
 #endif
