@@ -194,9 +194,13 @@ gog_pie_plot_set_property (GObject *obj, guint param_id,
 	GogPiePlot *pie = GOG_PIE_PLOT (obj);
 
 	switch (param_id) {
-	case PLOT_PROP_INITIAL_ANGLE:
-		pie->initial_angle = g_value_get_double (value);
+	case PLOT_PROP_INITIAL_ANGLE: {
+		double a = g_value_get_double (value);
+		a = fmod (a, 360);
+		if (a < 0) a += 360;
+		pie->initial_angle = a;
 		break;
+	}
 	case PLOT_PROP_DEFAULT_SEPARATION: {
 		double f = g_value_get_double (value);
 		pie->default_separation = CLAMP (f, 0., 5.);
@@ -306,7 +310,7 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 		g_param_spec_double ("initial-angle",
 			_("Initial angle"),
 			_("Degrees clockwise from 12 O'Clock."),
-			0, G_MAXFLOAT, 0.,
+			-G_MAXFLOAT, G_MAXFLOAT, 0.,
 			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_DEFAULT_SEPARATION,
 		g_param_spec_double ("default-separation",
@@ -461,7 +465,7 @@ gog_ring_plot_class_init (GogPiePlotClass *pie_plot_klass)
 			_("Center-size"),
 			_("Size of the center hole as a percentage of the radius"),
 			0, 100., 0.,
-			GSF_PARAM_STATIC | G_PARAM_READWRITE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GO_PARAM_PERSISTENT));
 
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->desc.series.style_fields = GO_STYLE_OUTLINE | GO_STYLE_FILL;
@@ -644,7 +648,7 @@ gog_pie_view_get_data_at_point (GogPlotView *view, double x, double y, GogSeries
 	GogPieSeries const *pseries = NULL;
 	double r_tot, r_cur, r_int, r_ext, cx, cy, r, th, th0, theta, scale, *vals;
 	double separated_cx, separated_cy;
-	unsigned int index, elem, k;
+	unsigned int index, k;
 	double center_radius;
 	double center_size = 0.0;
 	unsigned num_series = 0;
@@ -787,7 +791,6 @@ gog_pie_view_get_data_at_point (GogPlotView *view, double x, double y, GogSeries
 	if (r < center_radius)
 		return -1;
 
-	elem = model->base.index_num;
 	index = 1;
 	th0 = (model->initial_angle + pseries->initial_angle) * M_PI / 180. - M_PI / 2.;
 	th = atan2 (y - cy, x - cx);
@@ -884,7 +887,7 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogPieSeries const *series = NULL;
 	double separated_cx, separated_cy, cx, cy, r, theta, len, *vals, scale;
 	double default_sep;
-	unsigned elem, k;
+	unsigned k;
 	GOPath *path;
 	GogTheme *theme = gog_object_get_theme (GOG_OBJECT (model));
 	GOStyle *style, *white_style = NULL, *elt_style = NULL;
@@ -1034,7 +1037,6 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	center_radius = r_tot * center_size;
 	r = r_tot * (1. - center_size);
 
-	elem = model->base.index_num;
 	index = 1;
 	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
 		series = ptr->data;
